==> syncfusion_flutter_barcodes-18.1.36-beta.tar.gz: lib/src/barcode_generator/one_dimensional/ean8_symbology.dart <==
part of barcodes;

/// Represents the EAN-8 symbology
class EAN8 extends Symbology {
  /// Creates the EAN-8 symbology
  EAN8({int module})
      : super(
          module: module,
        );

  /// Represents the encoded input value
  String _encodedValue;

  @override
  bool _validateInput(String value) {
    if (value.contains(RegExp(r'^(?=.*?[0-9]).{8}$')) &&
        int.parse(value[7]) == _checkSumData(value)) {
      _encodedValue = value;
    } else if (value.contains(RegExp(r'^(?=.*?[0-9]).{7}$'))) {
      _encodedValue = value + _checkSumData(value).toString();
    } else {
      throw 'EAN8 supports only numeric characters. The provided value should have'
          ' 7 digits (without checksum) or  with 8 digits';
    }
    return true;
  }

  @override
  void _renderBarcode(
      Canvas canvas,
      Size size,
      Offset offset,
      String value,
      Color foregroundColor,
      TextStyle textStyle,
      double textSpacing,
      TextAlign textAlign,
      bool showValue) {
    final Paint _paint = _getBarPaint(foregroundColor);
    final List<String> code = _getCodeValues();
    final int _barTotalLength = _calculateTotalLength(code);
    double _left = module == null
        ? offset.dx
        : _calculateLeftPosition(
            _barTotalLength, module, size.width, offset.dx);
    final Rect _barCodeRect = Rect.fromLTRB(
        offset.dx, offset.dy, offset.dx + size.width, offset.dy + size.height);

    double _ratio = 0;
    if (module != null) {
      _ratio = module.toDouble();
    } else {
      final int singleModule = (size.width ~/ _barTotalLength).toInt();
      _ratio = singleModule.toDouble();
      final double _leftPadding = (size.width - (_barTotalLength * _ratio)) / 2;
      _left += _leftPadding;
    }
    _left = _left.roundToDouble();
    for (int i = 0; i < code.length; i++) {
      final String _codeValue = code[i];
      const bool _hasExtraHeight = false;
      final double _barHeight = _hasExtraHeight
          ? size.height + _textSize.height + textSpacing
          : size.height;
      final int _codeLength = _codeValue.length;
      for (int j = 0; j < _codeLength; j++) {
        final bool _canDraw = _codeValue[j] == '1' ? true : false;
        if (_canDraw &&
            (_left >= _barCodeRect.left &&
                _left + _ratio < _barCodeRect.right)) {
          final Rect _individualBarRect = Rect.fromLTRB(
              _left, offset.dy, _left + _ratio, offset.dy + _barHeight);
          canvas.drawRect(_individualBarRect, _paint);
        }
        _left += _ratio;
      }
    }
    if (showValue)
      _drawText(canvas, offset, size, value, textStyle, textSpacing, textAlign);
  }

  /// Calculate total bar length from given input value
  int _calculateTotalLength(List<String> code) {
    int _count = 0;
    for (int i = 0; i < code.length; i++) {
      final int _numberOfDigits = code[i].length;
      _count += _numberOfDigits;
    }
    return _count;
  }

  /// Returns the encoded value
  List<String> _getCodeValues() {
    const String _endBars = '101';
    const String _middleBar = '01010';
    Map<String, String> _codes = _getCodeValueRight(true);
    final List<String> _code = <String>[];
    _code.add(_endBars);
    _code.add(_getLeftValue(_codes, true));
    _code.add(_middleBar);
    _codes = _getCodeValueRight(false);
    _code.add(_getLeftValue(_codes, false));
    _code.add(_endBars);
    return _code;
  }

  /// Represents the encoded value for the first 6 digits of the input value
  String _getLeftValue(Map<String, String> _codes, bool _isLeft) {
    String _code = '';
    for (int i = _isLeft ? 0 : _encodedValue.length - 4;
        i < (_isLeft ? _encodedValue.length - 4 : _encodedValue.length);
        i++) {
      final int _currentValue = int.parse(_encodedValue[i]);
      if (i == 0 || i == 4) {
        _code = _codes.entries.elementAt(_currentValue).value;
      } else {
        _code += _codes.entries.elementAt(_currentValue).value;
      }
    }
    return _code;
  }

  /// Method to calculate the input data
  int _checkSumData(String _value) {
    for (int i = 0; i < _value.length; i++) {
      final int _sum1 =
          int.parse(_value[1]) + int.parse(_value[3]) + int.parse(_value[5]);
      final int _sum2 = 3 *
          (int.parse(_value[0]) +
              int.parse(_value[2]) +
              int.parse(_value[4]) +
              int.parse(_value[6]));
      final int _checkSumValue = _sum1 + _sum2;
      final int _checkSumDigit = (10 - _checkSumValue) % 10;
      return _checkSumDigit;
    }
    return 0;
  }

  /// Represents the encoded value for the last 6 digits of the input value
  Map<String, String> _getCodeValueRight(bool _isRight) {
    Map<String, String> _codes;
    if (_isRight) {
      _codes = <String, String>{
        '0': '0001101',
        '1': '0011001',
        '2': '0010011',
        '3': '0111101',
        '4': '0100011',
        '5': '0110001',
        '6': '0101111',
        '7': '0111011',
        '8': '0110111',
        '9': '0001011',
      };
    } else {
      _codes = <String, String>{
        '0': '1110010',
        '1': '1100110',
        '2': '1101100',
        '3': '1000010',
        '4': '1011100',
        '5': '1001110',
        '6': '1010000',
        '7': '1000100',
        '8': '1001000',
        '9': '1110100'
      };
    }
    return _codes;
  }
}

==> syncfusion_flutter_barcodes-18.1.43-beta.tar.gz: lib/src/barcode_generator/one_dimensional/ean8_symbology.dart <==
part of barcodes;

/// The [EAN8] is equivalent to the [UPCE] for small packaging details.
/// It is shorter than the [EAN13] barcode and longer than [UPCE].
///
/// It encodes 7 digits of numeric data with the check digit at its end.
///
class EAN8 extends Symbology {
  /// Create a [EAN8] symbology with the default or required properties.
  ///
  /// The arguments [module] must be non-negative and greater than 0.
  ///
  /// [EAN8] allows either 7 or 8 digits of numeric data otherwise there is an exception.
  ///
  /// If the input is 7 digits of the numeric data, the check digit is calculated automatically.
  ///
  /// If the input is 8 digits of the numeric data, the last digit must be valid check digit otherwise remove it, since it has been calculated automatically.
  ///
  EAN8({int module})
      : super(
          module: module,
        );

  /// Represents the encoded input value
  String _encodedValue;

  @override
  bool _validateInput(String value) {
    if (value.contains(RegExp(r'^(?=.*?[0-9]).{8}$'))) {
      if (int.parse(value[7]) == _checkSumData(value)) {
        _encodedValue = value;
      } else {
        throw 'Invalid check digit at the trailing end.'
            ' Provide valid check digit or remove it.'
            ' Since it has been calculated automatically.';
      }
    } else if (value.contains(RegExp(r'^(?=.*?[0-9]).{7}$'))) {
      _encodedValue = value + _checkSumData(value).toString();
    } else {
      throw 'EAN8 supports only numeric characters. The provided value should have'
          ' 7 digits (without check digit) or  with 8 digits';
    }
    return true;
  }

  @override
  void _renderBarcode(
      Canvas canvas,
      Size size,
      Offset offset,
      String value,
      Color foregroundColor,
      TextStyle textStyle,
      double textSpacing,
      TextAlign textAlign,
      bool showValue) {
    ///  _positions[0] specifies end position of start  bar
    ///  _positions[1] specifies start position of middle  bar
    ///  _positions[2] specifies end position of middle bar
    ///  _positions[3] specifies start position of end bar
    final List<double> _positions = List<double>(4);
    final Paint _paint = _getBarPaint(foregroundColor);
    final List<String> code = _getCodeValues();
    final int _barTotalLength = _calculateTotalLength(code);
    double _left = module == null
        ? offset.dx
        : _calculateLeftPosition(
            _barTotalLength, module, size.width, offset.dx);
    final Rect _barCodeRect = Rect.fromLTRB(
        offset.dx, offset.dy, offset.dx + size.width, offset.dy + size.height);

    double _ratio = 0;
    if (module != null) {
      _ratio = module.toDouble();
    } else {
      final int singleModule = (size.width ~/ _barTotalLength).toInt();
      _ratio = singleModule.toDouble();
      final double _leftPadding = (size.width - (_barTotalLength * _ratio)) / 2;
      _left += _leftPadding;
    }
    _left = _left.roundToDouble();
    for (int i = 0; i < code.length; i++) {
      final String _codeValue = code[i];
      final bool _hasExtraHeight = _checkExtraHeight(i, code);
      final double _additionalHeight = i == 2 ? 0.4 : 0.5;
      final double _barHeight = _hasExtraHeight
          ? size.height +
              (showValue
                  ? (_textSize.height * _additionalHeight) + textSpacing
                  : 0)
          : size.height;
      final int _codeLength = _codeValue.length;
      for (int j = 0; j < _codeLength; j++) {
        final bool _canDraw = _codeValue[j] == '1' ? true : false;
        if (_canDraw &&
            (_left >= _barCodeRect.left &&
                _left + _ratio < _barCodeRect.right)) {
          final Rect _individualBarRect = Rect.fromLTRB(
              _left, offset.dy, _left + _ratio, offset.dy + _barHeight);
          canvas.drawRect(_individualBarRect, _paint);
        }
        _left += _ratio;
        if (i == 0 && j == _codeLength - 1) {
          _positions[0] = _left;
        } else if (i == 1 && j == _codeLength - 1) {
          _positions[1] = _left;
        } else if (i == 2 && j == _codeLength - 1) {
          _positions[2] = _left;
        } else if (i == 3 && j == _codeLength - 1) {
          _positions[3] = _left;
        }
      }
    }
    if (showValue)
      _paintText(canvas, offset, size, _encodedValue, textStyle, textSpacing,
          textAlign, _positions);
  }

  /// Calculate total bar length from given input value
  int _calculateTotalLength(List<String> code) {
    int _count = 0;
    for (int i = 0; i < code.length; i++) {
      final int _numberOfDigits = code[i].length;
      _count += _numberOfDigits;
    }
    return _count;
  }

  /// Returns the encoded value
  List<String> _getCodeValues() {
    const String _endBars = '101';
    const String _middleBar = '01010';
    Map<String, String> _codes = _getCodeValueRight(true);
    final List<String> _code = <String>[];
    _code.add(_endBars);
    _code.add(_getLeftValue(_codes, true));
    _code.add(_middleBar);
    _codes = _getCodeValueRight(false);
    _code.add(_getLeftValue(_codes, false));
    _code.add(_endBars);
    return _code;
  }

  /// Represents the encoded value for the first 6 digits of the input value
  String _getLeftValue(Map<String, String> _codes, bool _isLeft) {
    String _code = '';
    for (int i = _isLeft ? 0 : _encodedValue.length - 4;
        i < (_isLeft ? _encodedValue.length - 4 : _encodedValue.length);
        i++) {
      final int _currentValue = int.parse(_encodedValue[i]);
      if (i == 0 || i == 4) {
        _code = _codes.entries.elementAt(_currentValue).value;
      } else {
        _code += _codes.entries.elementAt(_currentValue).value;
      }
    }
    return _code;
  }

  /// Method to calculate the input data
  int _checkSumData(String _value) {
    for (int i = 0; i < _value.length; i++) {
      final int _sum1 =
          int.parse(_value[1]) + int.parse(_value[3]) + int.parse(_value[5]);
      final int _sum2 = 3 *
          (int.parse(_value[0]) +
              int.parse(_value[2]) +
              int.parse(_value[4]) +
              int.parse(_value[6]));
      final int _checkSumValue = _sum1 + _sum2;
      final int _checkSumDigit = (10 - _checkSumValue) % 10;
      return _checkSumDigit;
    }
    return 0;
  }

  /// Represents the encoded value for the last 6 digits of the input value
  Map<String, String> _getCodeValueRight(bool _isRight) {
    Map<String, String> _codes;
    if (_isRight) {
      _codes = <String, String>{
        '0': '0001101',
        '1': '0011001',
        '2': '0010011',
        '3': '0111101',
        '4': '0100011',
        '5': '0110001',
        '6': '0101111',
        '7': '0111011',
        '8': '0110111',
        '9': '0001011',
      };
    } else {
      _codes = <String, String>{
        '0': '1110010',
        '1': '1100110',
        '2': '1101100',
        '3': '1000010',
        '4': '1011100',
        '5': '1001110',
        '6': '1010000',
        '7': '1000100',
        '8': '1001000',
        '9': '1110100'
      };
    }
    return _codes;
  }

  /// Method to render the input value of the barcode
  void _paintText(
      Canvas canvas,
      Offset offset,
      Size size,
      String value,
      TextStyle textStyle,
      double textSpacing,
      TextAlign textAlign,
      List<double> _positions) {
    final String _value1 = value.substring(0, 4);
    final String _value2 = value.substring(4, 8);
    final double _firstTextWidth = _positions[1] - _positions[0];
    final double _secondTextWidth = _positions[3] - _positions[2];

    _drawText(
        canvas,
        Offset(_positions[0], offset.dy),
        Size(_firstTextWidth, size.height),
        _value1,
        textStyle,
        textSpacing,
        textAlign,
        offset,
        size);
    _drawText(
        canvas,
        Offset(_positions[2], offset.dy),
        Size(_secondTextWidth, size.height),
        _value2,
        textStyle,
        textSpacing,
        textAlign,
        offset,
        size);
  }
}
